@@ -1,5 +1,5 @@
-# Copyright (C) 2007 Canonical Ltd
-# Copyright (C) 2009-2010 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2007,2011 Canonical Ltd
+# Copyright (C) 2009-2011 Jelmer Vernooij <jelmer@samba.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 """An adapter between a Git Branch and a Bazaar Branch"""
 
+from cStringIO import StringIO
 from collections import defaultdict
 
 from dulwich.objects import (
@@ -269,7 +270,7 @@ class GitSymrefBranchFormat(branch.BranchFormat):
         return controldir.get_branch_reference(name)
 
     def set_reference(self, controldir, name, target):
-        return controldir.set_branch_reference(name, target)
+        return controldir.set_branch_reference(target, name)
 
 
 class GitBranchFormat(branch.BranchFormat):
@@ -380,10 +381,19 @@ class GitBranch(ForeignBranch):
 
         :return: Branch nick
         """
-        return self.name or "HEAD"
+        cs = self.repository._git.get_config_stack()
+        try:
+            return cs.get(("branch", self.name), "nick")
+        except KeyError:
+            pass
+        return self.name.encode('utf-8') or "HEAD"
 
     def _set_nick(self, nick):
-        raise NotImplementedError
+        cf = self.repository._git.get_config()
+        cf.set(("branch", self.name), "nick", nick)
+        f = StringIO()
+        cf.write_to_file(f)
+        self.bzrdir.control_transport.put_bytes('config', f.getvalue())
 
     nick = property(_get_nick, _set_nick)
 
@@ -633,7 +643,7 @@ class GitBranchPullResult(branch.PullResult):
 
     def _lookup_revno(self, revid):
         return _quick_lookup_revno(self.target_branch, self.source_branch,
-                revid)
+            revid)
 
     def _get_old_revno(self):
         if self._old_revno is not None:
@@ -905,6 +915,15 @@ class InterGitLocalGitBranch(InterGitBranch):
         return (isinstance(source, GitBranch) and
                 isinstance(target, LocalGitBranch))
 
+    def fetch(self, stop_revision=None, fetch_tags=None, limit=None):
+        interrepo = _mod_repository.InterRepository.get(self.source.repository,
+            self.target.repository)
+        if stop_revision is None:
+            stop_revision = self.source.last_revision()
+        determine_wants = interrepo.get_determine_wants_revids(
+            [stop_revision], include_tags=fetch_tags)
+        interrepo.fetch_objects(determine_wants, limit=limit)
+
     def _basic_push(self, overwrite=False, stop_revision=None):
         result = GitBranchPushResult()
         result.source_branch = self.source
@@ -1073,7 +1092,7 @@ class InterToGitBranch(branch.GenericInterBranch):
             for k, v in self.source.tags.get_tag_dict().iteritems():
                 ret.append((None, v))
         ret.append((None, stop_revision))
-        self.interrepo.fetch_objects(ret, roundtrip=(not lossy))
+        self.interrepo.fetch_objects(ret, lossy=lossy)
 
     def pull(self, overwrite=False, stop_revision=None, local=False,
              possible_transports=None, run_hooks=True):
@@ -1093,7 +1112,7 @@ class InterToGitBranch(branch.GenericInterBranch):
                         update_refs, lossy=False)
                 except NoPushSupport:
                     raise errors.NoRoundtrippingSupport(self.source, self.target)
-                (result.old_revid, old_sha1) = old_refs.get(main_ref, (ZERO_SHA, NULL_REVISION))
+                (old_sha1, result.old_revid) = old_refs.get(main_ref, (ZERO_SHA, NULL_REVISION))
                 if result.old_revid is None:
                     result.old_revid = self.target.lookup_foreign_revision_id(old_sha1)
                 result.new_revid = new_refs[main_ref][1]
@@ -1136,10 +1155,6 @@ class InterToGitBranch(branch.GenericInterBranch):
             self.source.unlock()
         return result
 
-    def lossy_push(self, stop_revision=None):
-        # For compatibility with bzr < 2.4
-        return self.push(lossy=True, stop_revision=stop_revision)
-
 
 branch.InterBranch.register_optimiser(InterGitLocalGitBranch)
 branch.InterBranch.register_optimiser(InterFromGitBranch)
