@@ -77,22 +77,25 @@ class GitDir(ControlDir):
     def checkout_metadir(self, stacked=False):
         return format_registry.make_bzrdir("default")
 
+    def _get_default_ref(self):
+        return "HEAD"
+
     def _get_selected_ref(self, branch, ref=None):
         if ref is not None and branch is not None:
             raise bzr_errors.BzrError("can't specify both ref and branch")
         if ref is not None:
             return ref
-        if branch is None and getattr(self, "_get_selected_branch", False):
-            branch = self._get_selected_branch()
-        if branch is not None:
-            from bzrlib.plugins.git.refs import branch_name_to_ref
-            return branch_name_to_ref(branch, None)
         segment_parameters = getattr(
             self.user_transport, "get_segment_parameters", lambda: {})()
         ref = segment_parameters.get("ref")
         if ref is not None:
-            ref = urlutils.unescape(ref)
-        return ref
+            return urlutils.unescape(ref)
+        if branch is None and getattr(self, "_get_selected_branch", False):
+            branch = self._get_selected_branch()
+        if branch is not None:
+            from bzrlib.plugins.git.refs import branch_name_to_ref
+            return branch_name_to_ref(branch)
+        return self._get_default_ref()
 
     def get_config(self):
         return GitDirConfig()
@@ -127,7 +130,7 @@ class GitDir(ControlDir):
 
         if revision_id is not None:
             determine_wants = interrepo.get_determine_wants_revids(
-                [revision_id], include_tags=True)
+                [revision_id], include_tags=False)
         else:
             determine_wants = interrepo.determine_wants_all
         interrepo.fetch_objects(determine_wants=determine_wants,
@@ -222,7 +225,8 @@ class LocalGitControlDirFormat(GitControlDirFormat):
 
     def initialize_on_transport(self, transport):
         from bzrlib.plugins.git.transportgit import TransportRepo
-        TransportRepo.init(transport, bare=self.bare)
+        repo = TransportRepo.init(transport, bare=self.bare)
+        del repo.refs["HEAD"]
         return self.open(transport)
 
     def initialize_on_transport_ex(self, transport, use_existing_dir=False,
@@ -316,18 +320,14 @@ class LocalGitDir(GitDir):
             return refcontents[len(SYMREF):].rstrip("\n")
         return None
 
-    def set_branch_reference(self, name, target):
+    def set_branch_reference(self, target, name=None):
+        if self.control_transport.base != target.bzrdir.control_transport.base:
+            raise bzr_errors.IncompatibleFormat(target._format, self._format)
         ref = self._get_selected_ref(name)
-        if ref is None:
-            ref = "HEAD"
-        if not getattr(target, "ref", None):
-            raise bzr_errors.BzrError("Can only set symrefs to Git refs")
         self._git.refs.set_symbolic_ref(ref, target.ref)
 
     def get_branch_reference(self, name=None):
         ref = self._get_selected_ref(name)
-        if ref is None:
-            ref = "HEAD"
         target_ref = self._get_symref(ref)
         if target_ref is not None:
             return urlutils.join_segment_parameters(
@@ -340,8 +340,6 @@ class LocalGitDir(GitDir):
             GitSymrefBranchFormat,
             )
         ref = self._get_selected_ref(name)
-        if ref is None:
-            ref = "HEAD"
         if self._get_symref(ref) is not None:
             return GitSymrefBranchFormat()
         else:
@@ -374,8 +372,6 @@ class LocalGitDir(GitDir):
         repo = self.open_repository()
         from bzrlib.plugins.git.branch import LocalGitBranch
         ref = self._get_selected_ref(name, ref)
-        if ref is None:
-            ref = "HEAD"
         ref, sha = self._git.refs._follow(ref)
         if not ref in self._git.refs:
             raise bzr_errors.NotBranchError(self.root_transport.base,
@@ -384,8 +380,6 @@ class LocalGitDir(GitDir):
 
     def destroy_branch(self, name=None):
         refname = self._get_selected_ref(name)
-        if refname is None:
-            refname = "refs/heads/master"
         try:
             del self._git.refs[refname]
         except KeyError:
@@ -422,7 +416,7 @@ class LocalGitDir(GitDir):
         """'open' a repository for this dir."""
         return self._gitrepository_class(self)
 
-    def open_workingtree(self, recommend_upgrade=True):
+    def open_workingtree(self, recommend_upgrade=True, unsupported=False):
         if not self._git.bare:
             from dulwich.errors import NoIndexPresent
             repo = self.open_repository()
@@ -451,31 +445,23 @@ class LocalGitDir(GitDir):
                       append_revisions_only=None, ref=None):
         refname = self._get_selected_ref(name, ref)
         from dulwich.protocol import ZERO_SHA
-        # FIXME: This is a bit awkward. Perhaps we should have a
-        # a separate method for changing the default branch?
-        if refname is None:
-            refname = "refs/heads/master"
-            set_head = True
-        else:
-            set_head = False
-
         if refname in self._git.refs:
-            raise bzr_errors.AlreadyBranchError(self.base)
+            raise bzr_errors.AlreadyBranchError(self.user_url)
         self._git.refs[refname] = ZERO_SHA
-        if set_head:
-            self._git.refs.set_symbolic_ref("HEAD", refname)
         branch = self.open_branch(name)
         if append_revisions_only:
             branch.set_append_revisions_only(append_revisions_only)
         return branch
 
     def backup_bzrdir(self):
-        if self._git.bare:
+        if not self._git.bare:
             self.root_transport.copy_tree(".git", ".git.backup")
             return (self.root_transport.abspath(".git"),
                     self.root_transport.abspath(".git.backup"))
         else:
-            raise bzr_errors.BzrError("Unable to backup bare repositories")
+            basename = urlutils.basename(self.root_transport.base)
+            parent = self.root_transport.clone('..')
+            parent.copy_tree(basename, basename + ".backup")
 
     def create_workingtree(self, revision_id=None, from_branch=None,
         accelerator_tree=None, hardlink=False):
