@@ -57,10 +57,7 @@ from bzrlib.repository import (
 from bzrlib.revision import (
     NULL_REVISION,
     )
-try:
-    from bzrlib.revisiontree import InventoryRevisionTree
-except ImportError: # bzr < 2.4
-    from bzrlib.revisiontree import RevisionTree as InventoryRevisionTree
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.testament import (
     StrictTestament3,
     )
@@ -420,10 +417,7 @@ def import_git_commit(repo, mapping, head, lookup_object,
     ret_tree = InventoryRevisionTree(repo, inv, rev.revision_id)
     # Check verifiers
     if verifiers and roundtrip_revid is not None:
-        if getattr(StrictTestament3, "from_revision_tree", None):
-            testament = StrictTestament3(rev, ret_tree)
-        else: # bzr < 2.4
-            testament = StrictTestament3(rev, inv)
+        testament = StrictTestament3(rev, ret_tree)
         calculated_verifiers = { "testament3-sha1": testament.as_sha1() }
         if calculated_verifiers != verifiers:
             trace.mutter("Testament SHA1 %r for %r did not match %r.",
@@ -589,6 +583,8 @@ class InterGitNonGitRepository(InterFromGitRepository):
         potential = set()
         for k, v in refs.as_dict().iteritems():
             # For non-git target repositories, only worry about peeled
+            if v == ZERO_SHA:
+                continue
             potential.add(self.source.bzrdir.get_peeled(k))
         return list(potential - self._target_has_shas(potential))
 
@@ -600,6 +596,8 @@ class InterGitNonGitRepository(InterFromGitRepository):
                 for k, unpeeled in refs.as_dict().iteritems():
                     if not is_tag(k):
                         continue
+                    if unpeeled == ZERO_SHA:
+                        continue
                     potential.add(self.source.bzrdir.get_peeled(k))
             return list(potential - self._target_has_shas(potential))
         return determine_wants
@@ -774,7 +772,7 @@ class InterGitGitRepository(InterFromGitRepository):
         old_refs = self.target.bzrdir.get_refs_container()
         ref_changes = {}
         def determine_wants(heads):
-            old_refs = dict([(k, (v, None)) for (k, v) in heads.iteritems()])
+            old_refs = dict([(k, (v, None)) for (k, v) in heads.as_dict().iteritems()])
             new_refs = update_refs(old_refs)
             ref_changes.update(new_refs)
             return [sha1 for (sha1, bzr_revid) in new_refs.itervalues()]
@@ -784,7 +782,7 @@ class InterGitGitRepository(InterFromGitRepository):
         new_refs = self.target.bzrdir.get_refs_container()
         return None, old_refs, new_refs
 
-    def fetch_objects(self, determine_wants, mapping=None):
+    def fetch_objects(self, determine_wants, mapping=None, limit=None):
         graphwalker = self.target._git.get_graph_walker()
         if (isinstance(self.source, LocalGitRepository) and
             isinstance(self.target, LocalGitRepository)):
@@ -824,7 +822,7 @@ class InterGitGitRepository(InterFromGitRepository):
         return set([sha for sha in shas if sha in self.target._git.object_store])
 
     def fetch(self, revision_id=None, find_ghosts=False,
-              mapping=None, fetch_spec=None, branches=None):
+              mapping=None, fetch_spec=None, branches=None, limit=None):
         if mapping is None:
             mapping = self.source.get_mapping()
         r = self.target._git
@@ -865,7 +863,7 @@ class InterGitGitRepository(InterFromGitRepository):
             include_tags=include_tags)
 
     def determine_wants_all(self, refs):
-        potential = set(refs.as_dict().values())
+        potential = set([v for v in refs.as_dict().values() if not v == ZERO_SHA])
         return list(potential - self._target_has_shas(potential))
 
     def get_determine_wants_heads(self, wants, include_tags=False):
@@ -876,6 +874,8 @@ class InterGitGitRepository(InterFromGitRepository):
                 for k, unpeeled in refs.as_dict().iteritems():
                     if not is_tag(k):
                         continue
+                    if unpeeled == ZERO_SHA:
+                        continue
                     potential.add(unpeeled)
             return list(potential - self._target_has_shas(potential))
         return determine_wants
