@@ -38,6 +38,7 @@ from bzrlib.foreign import (
 from bzrlib.revision import (
     NULL_REVISION,
     )
+from bzrlib.plugins.git.errors import NoPushSupport
 from bzrlib.plugins.git.hg import (
     format_hg_metadata,
     extract_hg_metadata,
@@ -45,7 +46,7 @@ from bzrlib.plugins.git.hg import (
 from bzrlib.plugins.git.roundtrip import (
     extract_bzr_metadata,
     inject_bzr_metadata,
-    BzrGitRevisionMetadata,
+    CommitSupplement,
     deserialize_fileid_map,
     serialize_fileid_map,
     )
@@ -54,7 +55,7 @@ DEFAULT_FILE_MODE = stat.S_IFREG | 0644
 
 
 def escape_file_id(file_id):
-    return file_id.replace('_', '__').replace(' ', '_s')
+    return file_id.replace('_', '__').replace(' ', '_s').replace('\x0c', '_c')
 
 
 def unescape_file_id(file_id):
@@ -68,6 +69,8 @@ def unescape_file_id(file_id):
                 ret.append("_")
             elif file_id[i+1] == 's':
                 ret.append(" ")
+            elif file_id[i+1] == 'c':
+                ret.append("\x0c")
             else:
                 raise AssertionError("unknown escape character %s" %
                     file_id[i+1])
@@ -100,8 +103,11 @@ class BzrGitMapping(foreign.VcsMapping):
 
     BZR_DUMMY_FILE = None
 
+    def is_special_file(self, filename):
+        return (filename in (self.BZR_FILE_IDS_FILE, self.BZR_DUMMY_FILE))
+
     def __init__(self):
-        super(BzrGitMapping, self).__init__(foreign_git)
+        super(BzrGitMapping, self).__init__(foreign_vcs_git)
 
     def __eq__(self, other):
         return (type(self) == type(other) and
@@ -211,7 +217,7 @@ class BzrGitMapping(foreign.VcsMapping):
         return message, metadata
 
     def _decode_commit_message(self, rev, message, encoding):
-        return message.decode(encoding), BzrGitRevisionMetadata()
+        return message.decode(encoding), CommitSupplement()
 
     def _encode_commit_message(self, rev, message, encoding):
         return message.encode(encoding)
@@ -227,22 +233,22 @@ class BzrGitMapping(foreign.VcsMapping):
         b.set_raw_chunks(serialize_fileid_map(fileid_map))
         return b
 
-    def export_commit(self, rev, tree_sha, parent_lookup, roundtrip,
+    def export_commit(self, rev, tree_sha, parent_lookup, lossy,
                       verifiers):
         """Turn a Bazaar revision in to a Git commit
 
         :param tree_sha: Tree sha for the commit
         :param parent_lookup: Function for looking up the GIT sha equiv of a
             bzr revision
-        :param roundtrip: Whether to store roundtripping information.
+        :param lossy: Whether to store roundtripping information.
         :param verifiers: Verifiers info
         :return dulwich.objects.Commit represent the revision:
         """
         from dulwich.objects import Commit
         commit = Commit()
         commit.tree = tree_sha
-        if roundtrip:
-            metadata = BzrGitRevisionMetadata()
+        if not lossy:
+            metadata = CommitSupplement()
             metadata.verifiers = verifiers
         else:
             metadata = None
@@ -294,9 +300,12 @@ class BzrGitMapping(foreign.VcsMapping):
             for k, v in rev.properties.iteritems():
                 if not k in mapping_properties:
                     metadata.properties[k] = v
-        if self.roundtripping:
-            commit.message = inject_bzr_metadata(commit.message, metadata, 
-                                                 encoding)
+        if not lossy:
+            if self.roundtripping:
+                commit.message = inject_bzr_metadata(commit.message, metadata, 
+                                                     encoding)
+            else:
+                raise NoPushSupport()
         assert type(commit.message) == str
         return commit
 
@@ -318,7 +327,6 @@ class BzrGitMapping(foreign.VcsMapping):
             raise AssertionError("Commit object can't be None")
         rev = ForeignRevision(commit.id, self,
                 self.revision_id_foreign_to_bzr(commit.id))
-        rev.parent_ids = tuple([lookup_parent_revid(p) for p in commit.parents])
         rev.git_metadata = None
         def decode_using_encoding(rev, commit, encoding):
             rev.committer = str(commit.committer).decode(encoding)
@@ -349,6 +357,7 @@ class BzrGitMapping(foreign.VcsMapping):
             rev.properties['commit-timezone-neg-utc'] = ""
         rev.timestamp = commit.commit_time
         rev.timezone = commit.commit_timezone
+        rev.parent_ids = None
         if rev.git_metadata is not None:
             md = rev.git_metadata
             roundtrip_revid = md.revision_id
@@ -359,6 +368,8 @@ class BzrGitMapping(foreign.VcsMapping):
         else:
             roundtrip_revid = None
             verifiers = {}
+        if rev.parent_ids is None:
+            rev.parent_ids = tuple([lookup_parent_revid(p) for p in commit.parents])
         return rev, roundtrip_revid, verifiers
 
     def get_fileid_map(self, lookup_object, tree_sha):
@@ -433,6 +444,12 @@ mapping_registry.register_lazy('git-v1', "bzrlib.plugins.git.mapping",
     "BzrGitMappingv1")
 mapping_registry.register_lazy('git-experimental',
     "bzrlib.plugins.git.mapping", "BzrGitMappingExperimental")
+# Uncomment the next line to enable the experimental bzr-git mappings.
+# This will make sure all bzr metadata is pushed into git, allowing for
+# full roundtripping later.
+# NOTE: THIS IS EXPERIMENTAL. IT MAY EAT YOUR DATA OR CORRUPT
+# YOUR BZR OR GIT REPOSITORIES. USE WITH CARE.
+#mapping_registry.set_default('git-experimental')
 mapping_registry.set_default('git-v1')
 
 
@@ -462,7 +479,7 @@ class ForeignGit(ForeignVcs):
         return { "git commit": foreign_revid }
 
 
-foreign_git = ForeignGit()
+foreign_vcs_git = ForeignGit()
 default_mapping = mapping_registry.get_default()()
 
 
@@ -483,6 +500,8 @@ def mode_is_executable(mode):
 
 def mode_kind(mode):
     """Determine the Bazaar inventory kind based on Unix file mode."""
+    if mode is None:
+        return None
     entry_kind = (mode & 0700000) / 0100000
     if entry_kind == 0:
         return 'directory'
@@ -546,12 +565,11 @@ def directory_to_tree(entry, lookup_ie_sha1, unusual_modes, empty_file_name):
             mode = entry_mode(ie)
         hexsha = lookup_ie_sha1(ie)
         if hexsha is not None:
-            tree.add(mode, name.encode("utf-8"), hexsha)
+            tree.add(name.encode("utf-8"), mode, hexsha)
     if entry.parent_id is not None and len(tree) == 0:
         # Only the root can be an empty tree
         if empty_file_name is not None:
-            tree.add(stat.S_IFREG | 0644, empty_file_name, 
-                Blob().id)
+            tree.add(empty_file_name, stat.S_IFREG | 0644, Blob().id)
         else:
             return None
     return tree
@@ -580,6 +598,11 @@ class GitFileIdMap(object):
         self.paths = None
         self.mapping = mapping
 
+    def set_file_id(self, path, file_id):
+        assert type(path) is str
+        assert type(file_id) is str
+        self.file_ids[path] = file_id
+
     def lookup_file_id(self, path):
         assert type(path) is str
         try:
@@ -601,3 +624,6 @@ class GitFileIdMap(object):
         else:
             assert type(path) is str
             return path
+
+    def copy(self):
+        return self.__class__(dict(self.file_ids), self.mapping)
