@@ -49,11 +49,12 @@ class TestGitBlackBox(ExternalBase):
         return repo, builder.finish()[r1]
 
     def test_nick(self):
-        GitRepo.init(self.test_dir)
+        r = GitRepo.init(self.test_dir)
+        del r["HEAD"]
         dir = BzrDir.open(self.test_dir)
         dir.create_branch()
         output, error = self.run_bzr(['nick'])
-        self.assertEquals("master\n", output)
+        self.assertEquals("HEAD\n", output)
 
     def test_info(self):
         self.simple_commit()
@@ -103,9 +104,6 @@ class TestGitBlackBox(ExternalBase):
     def test_info_verbose(self):
         self.simple_commit()
 
-        if bzrlib_version < (2, 4):
-            self.knownFailure("bzr info uses inventory on bzr < 2.4")
-
         output, error = self.run_bzr(['info', '-v'])
         self.assertEqual(error, '')
         self.assertTrue("Standalone tree (format: git)" in output)
@@ -224,3 +222,26 @@ class TestGitBlackBox(ExternalBase):
         self.assertEquals(
             set([b.name for b in BzrDir.open("b").list_branches()]),
             set(["abranch", "bbranch"]))
+
+    def test_git_refs_from_git(self):
+        r = GitRepo.init("a", mkdir=True)
+        self.build_tree(["a/file"])
+        r.stage("file")
+        cid = r.do_commit(ref="refs/heads/abranch", committer="Joe <joe@example.com>", message="Dummy")
+        r["refs/tags/atag"] = cid
+        (stdout, stderr) = self.run_bzr(["git-refs", "a"])
+        self.assertEquals(stderr, "")
+        self.assertEquals(stdout,
+            'refs/tags/atag -> ' + cid + '\n'
+            'refs/heads/abranch -> ' + cid + '\n')
+
+    def test_git_refs_from_bzr(self):
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(["a/file"])
+        tree.add(["file"])
+        revid = tree.commit(committer="Joe <joe@example.com>", message="Dummy")
+        tree.branch.tags.set_tag("atag", revid)
+        (stdout, stderr) = self.run_bzr(["git-refs", "a"])
+        self.assertEquals(stderr, "")
+        self.assertTrue("refs/tags/atag -> " in stdout)
+        self.assertTrue("HEAD -> " in stdout)
