@@ -71,6 +71,12 @@ class FileidTests(tests.TestCase):
     def test_escape_space(self):
         self.assertEquals("bla_s", escape_file_id("bla "))
 
+    def test_escape_control_l(self):
+        self.assertEquals("bla_c", escape_file_id("bla\x0c"))
+
+    def test_unescape_control_l(self):
+        self.assertEquals("bla\x0c", unescape_file_id("bla_c"))
+
     def test_escape_underscore(self):
         self.assertEquals("bla__", escape_file_id("bla_"))
 
@@ -337,5 +343,5 @@ class DirectoryToTreeTests(tests.TestCase):
         b = Blob.from_string("bla")
         t1 = directory_to_tree(ie, lambda x: b.id, {}, None)
         t2 = Tree()
-        t2.add(0100644, "bar", b.id)
+        t2.add("bar", 0100644, b.id)
         self.assertEquals(t1, t2)
