@@ -346,6 +346,25 @@ class TransportRepo(BaseRepo):
         # missing index file, which is treated as empty.
         return not self.bare
 
+    def get_config(self):
+        from dulwich.config import ConfigFile
+        try:
+            return ConfigFile.from_file(self._controltransport.get('config'))
+        except NoSuchFile:
+            return ConfigFile()
+
+    def get_config_stack(self):
+        from dulwich.config import StackedConfig
+        backends = []
+        p = self.get_config()
+        if p is not None:
+            backends.append(p)
+            writable = p
+        else:
+            writable = None
+        backends.extend(StackedConfig.default_backends())
+        return StackedConfig(backends, writable=writable)
+
     def __repr__(self):
         return "<%s for %r>" % (self.__class__.__name__, self.transport)
 
