@@ -23,16 +23,22 @@ from cStringIO import (
     )
 from collections import defaultdict
 import errno
+from dulwich.errors import NotGitRepository
 from dulwich.index import (
     Index,
+    changes_from_tree,
+    cleanup_mode,
+    index_entry_from_stat,
     )
 from dulwich.object_store import (
     tree_lookup_path,
     )
 from dulwich.objects import (
     Blob,
+    S_IFGITLINK,
     ZERO_SHA,
     )
+from dulwich.repo import Repo
 import os
 import posixpath
 import stat
@@ -46,7 +52,6 @@ from bzrlib import (
     lock,
     osutils,
     trace,
-    transport,
     tree,
     workingtree,
     )
@@ -101,6 +106,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         if not self._lock_mode:
             self._lock_mode = 'r'
             self._lock_count = 1
+            self.index.read()
         else:
             self._lock_count += 1
         self.branch.lock_read()
@@ -110,6 +116,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         if not self._lock_mode:
             self._lock_mode = 'w'
             self._lock_count = 1
+            self.index.read()
         elif self._lock_mode == 'r':
             raise errors.ReadOnlyError(self)
         else:
@@ -121,6 +128,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         if not self._lock_mode:
             self._lock_mode = 'w'
             self._lock_count = 1
+            self.index.read()
         elif self._lock_mode == 'r':
             raise errors.ReadOnlyError(self)
         else:
@@ -137,12 +145,12 @@ class GitWorkingTree(workingtree.WorkingTree):
     def unlock(self):
         if not self._lock_count:
             return lock.cant_unlock_not_held(self)
+        self.branch.unlock()
         self._cleanup()
         self._lock_count -= 1
         if self._lock_count > 0:
             return
         self._lock_mode = None
-        self.branch.unlock()
 
     def _detect_case_handling(self):
         try:
@@ -159,6 +167,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         self.set_parent_ids([p for p, t in parents_list])
 
     def _index_add_entry(self, path, file_id, kind):
+        assert self._lock_mode is not None
         assert isinstance(path, basestring)
         assert type(file_id) == str or file_id is None
         if kind == "directory":
@@ -193,10 +202,8 @@ class GitWorkingTree(workingtree.WorkingTree):
         # Add an entry to the index or update the existing entry
         flags = 0 # FIXME
         encoded_path = path.encode("utf-8")
-        self.index[encoded_path] = (stat_val.st_ctime,
-                stat_val.st_mtime, stat_val.st_dev, stat_val.st_ino,
-                stat_val.st_mode, stat_val.st_uid, stat_val.st_gid,
-                stat_val.st_size, blob.id, flags)
+        self.index[encoded_path] = index_entry_from_stat(
+            stat_val, blob.id, flags)
         if self._versioned_dirs is not None:
             self._ensure_versioned_dir(encoded_path)
 
@@ -208,11 +215,13 @@ class GitWorkingTree(workingtree.WorkingTree):
         self._versioned_dirs.add(dirname)
 
     def _load_dirs(self):
+        assert self._lock_mode is not None
         self._versioned_dirs = set()
         for p in self.index:
             self._ensure_versioned_dir(posixpath.dirname(p))
 
     def _unversion_path(self, path):
+        assert self._lock_mode is not None
         encoded_path = path.encode("utf-8")
         try:
             del self.index[encoded_path]
@@ -421,17 +430,20 @@ class GitWorkingTree(workingtree.WorkingTree):
                 if not self.mapping.is_special_file(filename):
                     yield os.path.join(dir_relpath, filename)
 
+    @needs_read_lock
     def extras(self):
         """Yield all unversioned files in this WorkingTree.
         """
         return set(self._iter_files_recursive()) - set(self.index)
 
+    @needs_tree_write_lock
     def flush(self):
         # TODO: Maybe this should only write on dirty ?
         if self._lock_mode != 'w':
             raise errors.NotWriteLocked(self)
         self.index.write()
 
+    @needs_read_lock
     def __iter__(self):
         for path in self.index:
             yield self.path2id(path)
@@ -471,7 +483,9 @@ class GitWorkingTree(workingtree.WorkingTree):
         else:
             return True
 
+    @needs_read_lock
     def id2path(self, file_id):
+        assert type(file_id) is str, "file id not a string: %r" % file_id
         file_id = osutils.safe_utf8(file_id)
         path = self._fileid_map.lookup_path(file_id)
         # FIXME: What about directories?
@@ -541,6 +555,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         return self.repository.revision_tree(revid)
 
     def _is_versioned(self, path):
+        assert self._lock_mode is not None
         return (path in self.index or self._has_dir(path))
 
     def filter_unversioned_files(self, files):
@@ -586,6 +601,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         mode = stat_result.st_mode
         return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
+    @needs_read_lock
     def stored_kind(self, file_id, path=None):
         if path is None:
             path = self.id2path(file_id)
@@ -597,23 +613,26 @@ class GitWorkingTree(workingtree.WorkingTree):
                 return "directory"
             raise errors.NoSuchId(self, file_id)
 
-    if not osutils.supports_executable():
-        def is_executable(self, file_id, path=None):
+    def is_executable(self, file_id, path=None):
+        if getattr(self, "_supports_executable", osutils.supports_executable)():
+            if not path:
+                path = self.id2path(file_id)
+            mode = os.lstat(self.abspath(path)).st_mode
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+        else:
             basis_tree = self.basis_tree()
             if file_id in basis_tree:
                 return basis_tree.is_executable(file_id)
             # Default to not executable
             return False
-    else:
-        def is_executable(self, file_id, path=None):
-            if not path:
-                path = self.id2path(file_id)
-            mode = os.lstat(self.abspath(path)).st_mode
-            return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_stat
+    def _is_executable_from_path_and_stat(self, path, stat_result):
+        if getattr(self, "_supports_executable", osutils.supports_executable)():
+            return self._is_executable_from_path_and_stat_from_stat(path, stat_result)
+        else:
+            return self._is_executable_from_path_and_stat_from_basis(path, stat_result)
 
+    @needs_read_lock
     def list_files(self, include_root=False, from_dir=None, recursive=True):
         # FIXME: Yield non-versioned files
         if from_dir is None:
@@ -653,6 +672,7 @@ class GitWorkingTree(workingtree.WorkingTree):
                 ie = fk_entries[kind]()
                 yield path, "?", kind, None, ie
 
+    @needs_read_lock
     def all_file_ids(self):
         ids = {u"": self.path2id("")}
         for path in self.index:
@@ -669,6 +689,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         # FIXME: Check .gitsubmodules for path
         return False
 
+    @needs_read_lock
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         # FIXME: Is return order correct?
         if yield_parents:
@@ -711,6 +732,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         # The index just contains content, which won't have changed.
         self._reset_data()
 
+    @needs_read_lock
     def get_canonical_inventory_path(self, path):
         for p in self.index:
             if p.lower() == path.lower():
@@ -718,6 +740,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         else:
             return path
 
+    @needs_read_lock
     def _walkdirs(self, prefix=""):
         if prefix != "":
             prefix += "/"
@@ -742,6 +765,44 @@ class GitWorkingTree(workingtree.WorkingTree):
                 mode_kind(mode)))
         return per_dir.iteritems()
 
+    def _lookup_entry(self, path, update_index=False):
+        assert type(path) == str
+        entry = self.index[path]
+        index_mode = entry[-6]
+        index_sha = entry[-2]
+        disk_path = os.path.join(self.basedir, path)
+        disk_stat = os.lstat(disk_path)
+        disk_mtime = disk_stat.st_mtime
+        if isinstance(entry[1], tuple):
+            index_mtime = entry[1][0]
+        else:
+            index_mtime = int(entry[1])
+        mtime_delta = (disk_mtime - index_mtime)
+        disk_mode = cleanup_mode(disk_stat.st_mode)
+        if mtime_delta > 0 or disk_mode != index_mode:
+            if stat.S_ISDIR(disk_mode):
+                try:
+                    subrepo = Repo(disk_path)
+                except NotGitRepository:
+                    return (None, None)
+                else:
+                    disk_mode = S_IFGITLINK
+                    git_id = subrepo.head()
+            elif stat.S_ISLNK(disk_mode):
+                blob = Blob.from_string(os.readlink(disk_path).encode('utf-8'))
+                git_id = blob.id
+            elif stat.S_ISREG(disk_mode):
+                with open(disk_path, 'r') as f:
+                    blob = Blob.from_string(f.read())
+                git_id = blob.id
+            else:
+                raise AssertionError
+            if update_index:
+                flags = 0 # FIXME
+                self.index[path] = index_entry_from_stat(disk_stat, git_id, flags, disk_mode)
+            return (git_id, disk_mode)
+        return (index_sha, index_mode)
+
 
 class GitWorkingTreeFormat(workingtree.WorkingTreeFormat):
 
@@ -781,12 +842,15 @@ class InterIndexGitTree(tree.InterTree):
         return (isinstance(source, GitRevisionTree) and 
                 isinstance(target, GitWorkingTree))
 
+    @needs_read_lock
     def compare(self, want_unchanged=False, specific_files=None,
                 extra_trees=None, require_versioned=False, include_root=False,
                 want_unversioned=False):
-        changes = self._index.changes_from_tree(
+        # FIXME: Handle include_root
+        changes = changes_between_git_tree_and_index(
             self.source.store, self.source.tree, 
-            want_unchanged=want_unchanged)
+            self.target, want_unchanged=want_unchanged,
+            want_unversioned=want_unversioned)
         source_fileid_map = self.source._fileid_map
         target_fileid_map = self.target._fileid_map
         ret = tree_delta_from_git_changes(changes, self.target.mapping,
@@ -798,15 +862,30 @@ class InterIndexGitTree(tree.InterTree):
                     osutils.file_kind(self.target.abspath(e))))
         return ret
 
+    @needs_read_lock
     def iter_changes(self, include_unchanged=False, specific_files=None,
         pb=None, extra_trees=[], require_versioned=True,
         want_unversioned=False):
-        changes = self._index.changes_from_tree(
+        changes = changes_between_git_tree_and_index(
             self.source.store, self.source.tree,
-            want_unchanged=include_unchanged)
-        # FIXME: Handle want_unversioned
+            self.target, want_unchanged=include_unchanged,
+            want_unversioned=want_unversioned)
         return changes_from_git_changes(changes, self.target.mapping,
             specific_file=specific_files)
 
 
 tree.InterTree.register_optimiser(InterIndexGitTree)
+
+
+def changes_between_git_tree_and_index(object_store, tree, target,
+        want_unchanged=False, want_unversioned=False, update_index=False):
+    """Determine the changes between a git tree and a working tree with index.
+
+    """
+
+    names = target.index._byname.keys()
+    for (name, mode, sha) in changes_from_tree(names, target._lookup_entry,
+            object_store, tree, want_unchanged=want_unchanged):
+        if name == (None, None):
+            continue
+        yield (name, mode, sha)
